/*
 * Copyright (c) 2018 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;

/**
 * Purpose to test EncodingUtility class.
 *
 * @author Lars Johansson
 */
public class EncodingUtilityTest {

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringNull() {
        String s = null;
        assertThrows(NullPointerException.class, () -> { EncodingUtility.encode(s); });
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringEmpty() {
        String s = "";
        assertEquals(s, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringPercent() {
        String s = "%";
        String expected = "%25";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringPlus() {
        String s = "+";
        String expected = "%2B";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringUnderscore() {
        String s = "_";
        String expected ="_";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringWhitespace() {
        String s = " ";
        String expected ="+";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringNumbers() {
        String s = "12345678";
        assertEquals(s, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringLetters() {
        String s = "abcdefgh";
        assertEquals(s, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringLettersPlus() {
        String s = "abc+efgh";
        String expected = "abc%2Befgh";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringLettersWhitespace() {
        String s = "abcd fgh";
        String expected = "abcd+fgh";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testEncodeStringLettersPlusWhitespace() {
        String s = "ab de+gh";
        String expected = "ab+de%2Bgh";
        assertEquals(expected, EncodingUtility.encode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringNull() {
        String s = null;
        assertThrows(NullPointerException.class, () -> { EncodingUtility.decode(s); });

    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringEmpty() {
        String s = "";
        assertEquals(s, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringPercent() {
        String s = "%25";
        String expected = "%";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringPlus() {
        String s = "%2B";
        String expected = "+";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringUnderscore() {
        String s = "_";
        String expected = "_";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringWhitespace() {
        String s = "+";
        String expected = " ";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringNumbers() {
        String s = "12345678";
        assertEquals(s, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringLetters() {
        String s = "abcdefgh";
        assertEquals(s, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringLettersPlus() {
        String s = "abc%2Befgh";
        String expected = "abc+efgh";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringLettersWhitespace() {
        String s = "abcd+fgh";
        String expected = "abcd fgh";
        assertEquals(expected, EncodingUtility.decode(s));
    }

    /**
     * Test for encoding a string.
     */
    @Test
    void testDecodeStringLettersPlusWhitespace() {
        String s = "ab+de%2Bgh";
        String expected = "ab de+gh";
        assertEquals(expected, EncodingUtility.decode(s));
    }

}
