/*
 * Copyright (c) 2018 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Test;

/**
 * Purpose to test URLUtility class.
 *
 * @author Lars Johansson
 */
public class URLUtilityTest {

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLNull() {
        String url = null;
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLEmpty() {
        String url = "";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLWhitespace() {
        String url = " ";
        String expected = "+";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLString() {
        String url = "abcd1234";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPath() {
        String url = "abcd.xhtml";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQuery() {
        String url = "abcd.xhtml?efgh";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryPercent() {
        String url = "/abcd.xhtml?uvwx-0%";
        String expected = "/abcd.xhtml?uvwx-0%25";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryPlus() {
        String url = "abcd.xhtml?efg+h";
        String expected = "abcd.xhtml?efg%2Bh";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryWhitespace() {
        String url = "abcd.xhtml?ef gh";
        String expected = "abcd.xhtml?ef+gh";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue1Pair() {
        String url = "abcd.xhtml?efgh=ijkl";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue1PairEmpty() {
        String url = "abcd.xhtml?efgh=";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue1PairComplex() {
        String url = "abcd.xhtml?efgh=ijkl mnop+qrst+ uvwx";
        String expected = "abcd.xhtml?efgh=ijkl+mnop%2Bqrst%2B+uvwx";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue2Pairs() {
        String url = "abcd.xhtml?efgh=ijkl&mnop=qrst";
        assertEquals(url, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue2PairsComplex() {
        String url = "abcd.xhtml?efgh=i+kl&mnop=qr t";
        String expected = "abcd.xhtml?efgh=i%2Bkl&mnop=qr+t";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryAttributeValue3PairsComplex() {
        String url = "abcd.xhtml?efgh&uvwx=yzabc  def++ghijklm+ nop&abcdefgh=";
        String expected = "abcd.xhtml?efgh&uvwx=yzabc++def%2B%2Bghijklm%2B+nop&abcdefgh=";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

    /**
     * Test for encoding a URL.
     */
    @Test
    void testEncodeURLPathQueryIpPortPercent() {
        String url = "http://127.0.0.1:8080/a/b/c/d/e/f/RFQ-010:EMR-FS-0%";
        String expected = "http://127.0.0.1:8080/a/b/c/d/e/f/RFQ-010%3AEMR-FS-0%25";
        assertEquals(expected, URLUtility.encodeURL(url));
    }

}
